#ifdef _DEBUG
	#pragma comment( lib, "cv210d.lib" )
	#pragma comment( lib, "cxcore210d.lib" )
	#pragma comment( lib, "cvaux210d.lib" )
	#pragma comment( lib, "highgui210d.lib" )
#else
	#pragma comment( lib, "cv210.lib" )
	#pragma comment( lib, "cxcore210.lib" )
	#pragma comment( lib, "cvaux210.lib" )
	#pragma comment( lib, "highgui210.lib" )
#endif

#include <cv.h>
#include <cxcore.h>
#include <highgui.h>
#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <sstream>

int main( int argc, char** argv )
{
	const std::string						windowNameSrc = "Source";
	const std::string						windowNameUnd = "Undistorted Image";
	const int								numberOfCheckerPatterns = 5;

	// Lu[V
	cv::vector<cv::Mat>						checkerImgs;								// `FbJ[p^[摜
	const cv::Size							patternSize( 10, 7 );						// `FbJ[p^[̌_̐
	cv::vector<cv::vector<cv::Point3f>>		worldPoints( numberOfCheckerPatterns );		// `FbJ[_WƑΉ鐢EW̒li[s
	cv::vector<cv::vector<cv::Point2f>>		imagePoints( numberOfCheckerPatterns );		// `FbJ[_Wi[s
	cv::TermCriteria						criteria( CV_TERMCRIT_ITER | CV_TERMCRIT_EPS, 20, 0.001 );
	// Ή郏[hWnp^[

	// Jp[^s
	cv::Mat									cameraMatrix;		// p[^s
	cv::Mat									distCoeffs;			// Ycݍs
	cv::vector<cv::Mat>						rotationVectors;	// Be摜Ƃɓ]xNg
	cv::vector<cv::Mat>						translationVectors;	// Be摜Ƃɓ镽sړxNg

	// `FbJ[p^[摜ǂݍ
	for( int i = 0; i < numberOfCheckerPatterns; i++ ) {
		std::stringstream		stream;
		stream << "f" <<  i + 1 << ".jpg";
		std::string	fileName = stream.str();
		checkerImgs.push_back( cv::imread( fileName ) );
		std::cout << "Load checker image: " << fileName << std::endl;
	}

	// `FbNp^[̌_W߁CimagePointsɊi[
	cv::namedWindow( windowNameSrc, CV_WINDOW_AUTOSIZE );
	for( int i = 0; i < numberOfCheckerPatterns; i++ ) {
		std::cout << "Find corners from image " << i + 1;
		if( cv::findChessboardCorners( checkerImgs[i], patternSize, imagePoints[i] ) ) {
			std::cout << " ... All corners found." << std::endl;
			// o_`悷
			cv::drawChessboardCorners( checkerImgs[i], patternSize, ( cv::Mat )( imagePoints[i] ), true );
			cv::imshow( windowNameSrc, checkerImgs[i] );
			cv::waitKey( 500 );
		} else {
			std::cout << " ... at least 1 corner not found." << std::endl;
			cv::waitKey( 0 );
			return -1;
		}
	}

	// EW߂
	for( int i = 0; i < numberOfCheckerPatterns; i++ ) {
		for( int j = 0 ; j < patternSize.area(); j++ ) {
			worldPoints[i].push_back( cv::Point3f(	static_cast<float>( j % patternSize.width * 10 ), 
													static_cast<float>( j / patternSize.width * 10 ), 
													0.0 ) );
		}
	}

	// ܂ł̒lgăLu[V
	cv::calibrateCamera( worldPoints, imagePoints, checkerImgs[0].size(), cameraMatrix, distCoeffs, rotationVectors, translationVectors );
	std::cout << "Camera parameters have been estimated" << std::endl << std::endl;

	// cݕ␳摜\
	std::cout << "Undistorted images" << std::endl;
	cv::Mat	undistorted;
	cv::namedWindow( windowNameUnd );
	for( int i = 0; i < numberOfCheckerPatterns; i++ ) {
		cv::undistort( checkerImgs[i], undistorted, cameraMatrix, distCoeffs );
		cv::imshow( windowNameUnd, undistorted );
		cv::imshow( windowNameSrc, checkerImgs[i] );
		cv::waitKey( 0 );
	}

	return 0;
}